// For PPL renderer (r_ppl > 0)

_detailBump.vsh
{
	include bump.vsh
}

_detailBump.fsh
{
	uniform sampler2D	u_ColorMap;
	uniform float		u_ColorMapSwizzle;
	uniform sampler2D	u_NormalMap;
	uniform float		u_NormalMapSwizzle;
	uniform sampler2D	u_SpecularMap;
	uniform float		u_SpecularMapSwizzle;
	uniform sampler2D	u_detailNormalMap;
	uniform float		u_detailNormalMapSwizzle;
	uniform float		u_detailScale;
	uniform	float		u_Mirror;	// -1.0 for mirrors, else 1.0
	#if defined(ALPHATEST)
		uniform float	u_AlphaTestValue;
	#endif
	#if defined(CLIP)
		uniform vec4	u_clipPlane;
		in	vec3	v_clipVertex;
	#endif
	in	vec2		texCoord;
	in	vec3		Tangent;
	in	vec3		Binormal;
	in	vec3		Normal;
	in	vec4		matColor;
#if defined(FAKE_PPL)
		uniform sampler2D	u_LightMap;
		uniform float		u_useLightMap;
		uniform sampler2D	u_DeluxeMap;
		uniform float		u_useDeluxeMap;
		in	vec2		lmCoord;
		in	vec3		vertexColor;
#endif
	include Texture.inc
	include PackData.inc
	void main(void)
	{
		include clipPlane.inc
		vec4 pix = Texture(u_ColorMap, texCoord, u_ColorMapSwizzle) * matColor;
		include alpha_test.inc
		vec4 spec = Texture(u_SpecularMap, texCoord, u_SpecularMapSwizzle);
		vec3 nn = normalize(Normal) * mix(1.0, -1.0, float(gl_FrontFacing)) * u_Mirror;
		mat3 tangentToWorldMatrix = mat3(normalize(Tangent), normalize(Binormal), nn);
		vec4 tn = Texture(u_NormalMap, texCoord, u_NormalMapSwizzle);
		vec4 tdna = Texture(u_detailNormalMap, texCoord * u_detailScale, u_detailNormalMapSwizzle);
		vec3 tdn = tn.a * tdna.xyz * vec3(1.0, 1.0, 0.0);
		vec3 n = tangentToWorldMatrix * normalize(tn.xyz + tdn);
#if defined(FAKE_PPL)
		if (u_useDeluxeMap != 0.0)
			nn = texture(u_DeluxeMap, lmCoord).xyz * 2.0 - 1.0;
		if (u_useLightMap != 0.0)
			PackData(pix, n, spec.rgb * tdna.a, nn, texture(u_LightMap, lmCoord).rgb, spec.a);
		else
			PackData(pix, n, spec.rgb * tdna.a, nn, vertexColor, spec.a);
#else
		PackData(pix, n, spec.rgb * tdna.a, spec.a);
#endif
	}
}


_bump.vsh
{
	include bump.vsh
}

_bump.fsh
{
	uniform sampler2D	u_ColorMap;
	uniform float		u_ColorMapSwizzle;
	uniform sampler2D	u_NormalMap;
	uniform float		u_NormalMapSwizzle;
	uniform sampler2D	u_SpecularMap;
	uniform float		u_SpecularMapSwizzle;
	uniform	float		u_Mirror;	// -1.0 for mirrors, else 1.0
	#if defined(ALPHATEST)
		uniform float	u_AlphaTestValue;
	#endif
	#if defined(CLIP)
		uniform vec4	u_clipPlane;
		in	vec3	v_clipVertex;
	#endif
	in	vec2		texCoord;
	in	vec3		Tangent;
	in	vec3		Binormal;
	in	vec3		Normal;
	in	vec4		matColor;
#if defined(FAKE_PPL)
		uniform sampler2D	u_LightMap;
		uniform float		u_useLightMap;
		uniform sampler2D	u_DeluxeMap;
		uniform float		u_useDeluxeMap;
		in	vec2		lmCoord;
		in	vec3		vertexColor;
#endif
	include Texture.inc
	include PackData.inc
	void main(void)
	{
		include clipPlane.inc
		vec4 pix = Texture(u_ColorMap, texCoord, u_ColorMapSwizzle) * matColor;
		include alpha_test.inc
		vec4 spec = Texture(u_SpecularMap, texCoord, u_SpecularMapSwizzle);
		vec3 nn = normalize(Normal) * mix(1.0, -1.0, float(gl_FrontFacing)) * u_Mirror;
		mat3 tangentToWorldMatrix = mat3(normalize(Tangent), normalize(Binormal), nn);
		vec3 n = tangentToWorldMatrix * Texture(u_NormalMap, texCoord, u_NormalMapSwizzle).xyz;
#if defined(FAKE_PPL)
		if (u_useDeluxeMap != 0.0)
			nn = texture(u_DeluxeMap, lmCoord).xyz * 2.0 - 1.0;
		if (u_useLightMap != 0.0)
			PackData(pix, n, spec.rgb, nn, texture(u_LightMap, lmCoord).rgb, spec.a);
		else
			PackData(pix, n, spec.rgb, nn, vertexColor, spec.a);
#else
		PackData(pix, n, spec.rgb, spec.a);
#endif
	}
}


_bump_interpolate.vsh
{
	include bump_interpolate.vsh
}

_bump_interpolate.fsh
{
	uniform sampler2D	u_ColorMap0;
	uniform float		u_ColorMap0Swizzle;
	uniform sampler2D	u_ColorMap1;
	uniform float		u_ColorMap1Swizzle;
	uniform sampler2D	u_NormalMap0;
	uniform float		u_NormalMap0Swizzle;
	uniform sampler2D	u_NormalMap1;
	uniform float		u_NormalMap1Swizzle;
	uniform sampler2D	u_SpecularMap0;
	uniform float		u_SpecularMap0Swizzle;
	uniform sampler2D	u_SpecularMap1;
	uniform float		u_SpecularMap1Swizzle;
	uniform	float		u_Mirror;
	#if defined(CLIP)
		uniform vec4	u_clipPlane;
		in	vec3	v_clipVertex;
	#endif
	in	vec4		texCoords;
	in	vec3		Tangent;
	in	vec3		Binormal;
	in	vec3		Normal;
#if defined(FAKE_PPL)
		uniform sampler2D	u_LightMap;
		uniform float		u_useLightMap;
		uniform sampler2D	u_DeluxeMap;
		uniform float		u_useDeluxeMap;
		in	vec2		lmCoord;
		in	vec4		vertexColor;
#else
		in	float	vertexAlpha;
#endif
	include Texture.inc
	include PackData.inc
	void main(void)
	{
		include clipPlane.inc
		vec3 sample0 = Texture(u_ColorMap0, texCoords.xy, u_ColorMap0Swizzle).rgb;
		vec4 sample1 = Texture(u_ColorMap1, texCoords.zw, u_ColorMap1Swizzle);
#if defined(FAKE_PPL)
		float alpha = sample1.a * vertexColor.a;
#else
		float alpha = sample1.a * vertexAlpha;
#endif
		vec4 s0 = Texture(u_SpecularMap0, texCoords.xy, u_SpecularMap0Swizzle);
		vec4 s1 = Texture(u_SpecularMap1, texCoords.zw, u_SpecularMap1Swizzle);
		vec3 nn = normalize(Normal) * mix(1.0, -1.0, float(gl_FrontFacing)) * u_Mirror;
		mat3 tangentToWorldMatrix = mat3(normalize(Tangent), normalize(Binormal), nn);
		vec3 n0 = Texture(u_NormalMap0, texCoords.xy, u_NormalMap0Swizzle).xyz;
		vec3 n1 = Texture(u_NormalMap1, texCoords.zw, u_NormalMap1Swizzle).xyz;
		vec4 c = vec4(mix(sample0, sample1.rgb, alpha), 1.0);
		vec3 n = tangentToWorldMatrix * normalize(mix(n0, n1, alpha));
		vec4 spec = mix(s0, s1, alpha);
#if defined(FAKE_PPL)
		if (u_useDeluxeMap != 0.0)
			nn = texture(u_DeluxeMap, lmCoord).xyz * 2.0 - 1.0;
		if (u_useLightMap != 0.0)
			PackData(c, n, spec.rgb, nn, texture(u_LightMap, lmCoord).rgb, spec.a);
		else
			PackData(c, n, spec.rgb, nn, vertexColor.rgb, spec.a);
#else
		PackData(c, n, spec.rgb, spec.a);
#endif
	}
}


_bump_interpolate_at.vsh
{
	include bump_interpolate.vsh
}

_bump_interpolate_at.fsh
{
	uniform sampler2D	u_ColorMap0;
	uniform float		u_ColorMap0Swizzle;
	uniform sampler2D	u_ColorMap1;
	uniform float		u_ColorMap1Swizzle;
	uniform sampler2D	u_NormalMap0;
	uniform float		u_NormalMap0Swizzle;
	uniform sampler2D	u_NormalMap1;
	uniform float		u_NormalMap1Swizzle;
	uniform sampler2D	u_SpecularMap0;
	uniform float		u_SpecularMap0Swizzle;
	uniform sampler2D	u_SpecularMap1;
	uniform float		u_SpecularMap1Swizzle;
	uniform	float		u_Mirror;
	#if defined(CLIP)
		uniform vec4	u_clipPlane;
		in	vec3	v_clipVertex;
	#endif
	in	vec4		texCoords;
	in	vec3		Tangent;
	in	vec3		Binormal;
	in	vec3		Normal;
#if defined(FAKE_PPL)
		uniform sampler2D	u_LightMap;
		uniform float		u_useLightMap;
		uniform sampler2D	u_DeluxeMap;
		uniform float		u_useDeluxeMap;
		in	vec2		lmCoord;
		in	vec4		vertexColor;
#else
		in	float		vertexAlpha;
#endif
	include Texture.inc
	include PackData.inc
	void main(void)
	{
		include clipPlane.inc
		vec3 sample0 = Texture(u_ColorMap0, texCoords.xy, u_ColorMap0Swizzle).rgb;
		vec4 sample1 = Texture(u_ColorMap1, texCoords.zw, u_ColorMap1Swizzle);
#if defined(FAKE_PPL)
		float alpha = sample1.a * vertexColor.a;
#else
		float alpha = sample1.a * vertexAlpha;
#endif
		vec4 s0 = Texture(u_SpecularMap0, texCoords.xy, u_SpecularMap0Swizzle);
		vec4 s1 = Texture(u_SpecularMap1, texCoords.zw, u_SpecularMap1Swizzle);
		vec3 nn = normalize(Normal) * mix(1.0, -1.0, float(gl_FrontFacing)) * u_Mirror;
		mat3 tangentToWorldMatrix = mat3(normalize(Tangent), normalize(Binormal), nn);
		vec3 n0 = Texture(u_NormalMap0, texCoords.xy, u_NormalMap0Swizzle).xyz;
		vec3 n1 = Texture(u_NormalMap1, texCoords.zw, u_NormalMap1Swizzle).xyz;
		alpha *= step(0.5, alpha);
		vec4 c = vec4(mix(sample0, sample1.rgb, alpha), 1.0);
		vec3 n = tangentToWorldMatrix * normalize(mix(n0, n1, alpha));
		vec4 spec = mix(s0, s1, alpha);
#if defined(FAKE_PPL)
		if (u_useDeluxeMap != 0.0)
			nn = texture(u_DeluxeMap, lmCoord).xyz * 2.0 - 1.0;
		if (u_useLightMap != 0.0)
			PackData(c, n, spec.rgb, nn, texture(u_LightMap, lmCoord).rgb, spec.a);
		else
			PackData(c, n, spec.rgb, nn, vertexColor.rgb, spec.a);
#else
		PackData(c, n, spec.rgb, spec.a);
#endif
	}
}


_detailBump_parallax.vsh
{
	include parallax.vsh
}


_detailBump_parallax.fsh
{
	uniform sampler2D	u_ColorMap;
	uniform float		u_ColorMapSwizzle;
	uniform sampler2D	u_HeightMap;
	uniform float		u_HeightMapSwizzle;
	uniform sampler2D	u_NormalMap;
	uniform float		u_NormalMapSwizzle;
	uniform sampler2D	u_SpecularMap;
	uniform float		u_SpecularMapSwizzle;
	uniform sampler2D	u_detailNormalMap;
	uniform float		u_detailNormalMapSwizzle;
	uniform float		u_detailScale;
	uniform	float		u_Mirror;
	uniform vec2		u_texSize;
	include	lod.inc
	#if defined(ALPHATEST)
		uniform float	u_AlphaTestValue;
	#endif
	#if defined(CLIP)
		uniform vec4	u_clipPlane;
	#endif
	in	vec2	texCoord;
	in	vec3	Tangent;
	in	vec3	Binormal;
	in	vec3	Normal;
	in	vec3	eyeSpaceNormal;
	in	vec3	eyeSpaceVert;
	in	vec4	matColor;
#if defined(FAKE_PPL)
		uniform sampler2D	u_LightMap;
		uniform float		u_useLightMap;
		uniform sampler2D	u_DeluxeMap;
		uniform float		u_useDeluxeMap;
		in	vec2		lmCoord;
		in	vec3		vertexColor;
#endif
	include Texture.inc
#ifndef CSM
		in	vec3		eyeVec;
#else
		in	vec3		eyeSpaceTangent;
		in	vec3		eyeSpaceBinormal;
		include intersect_cone_exp.inc
#endif
	include PackData.inc
	void main(void)
	{
		vec3 v_clipVertex = eyeSpaceVert;
		include clipPlane.inc

		float a;
		vec3 pt_eye;
#if defined(CSM)
		vec3 T = normalize(eyeSpaceTangent);
		vec3 B = normalize(eyeSpaceBinormal);
#endif
		vec3 N = normalize(eyeSpaceNormal);
		vec3 v = normalize(eyeSpaceVert);

		a = abs(dot(N, v));
		float Scale = sin(clamp(a, 0.0, 1.0) * 1.570796327);

#if defined(CSM)
		vec3 s;
		float df;

		// ray intersect in view direction
		a = Scale * HEIGHT_SCALE / a;
		s = vec3(a*dot(T, v), a*dot(B, v), 1.0);

		df = 0.05 * sqrt(length(fwidth(texCoord)));

		// find the distance to the actualy heightfield
		pt_eye = vec3(texCoord, 0.0);
		intersect_cone_exp (pt_eye, s, df);
#else
		include linearStepParallax.inc
#endif
		// compute the final color
		vec4 pix = Texture(u_ColorMap, pt_eye.xy, u_ColorMapSwizzle) * matColor;
		include alpha_test.inc
		vec4 spec = Texture(u_SpecularMap, pt_eye.xy, u_SpecularMapSwizzle);
		vec3 nn = normalize(Normal) * mix(1.0, -1.0, float(gl_FrontFacing)) * u_Mirror;
		mat3 tangentToWorldMatrix = mat3(normalize(Tangent), normalize(Binormal), nn);
		vec4 tn = Texture(u_NormalMap, pt_eye.xy, u_NormalMapSwizzle);
		vec4 tdna = Texture(u_detailNormalMap, pt_eye.xy * u_detailScale, u_NormalMapSwizzle);
		vec3 tdn = tn.a * tdna.xyz * vec3(1.0, 1.0, 0.0);
		vec3 n = tangentToWorldMatrix * normalize(tn.xyz + tdn);
#if defined(FAKE_PPL)
		if (u_useDeluxeMap != 0.0)
			nn = texture(u_DeluxeMap, lmCoord).xyz * 2.0 - 1.0;
		if (u_useLightMap != 0.0)
			PackData(pix, n, spec.rgb * tdna.a, nn, texture(u_LightMap, lmCoord).rgb, spec.a);
		else
			PackData(pix, n, spec.rgb * tdna.a, nn, vertexColor, spec.a);
#else
		PackData(pix, n, spec.rgb * tdna.a, spec.a);
#endif
	}
}


_bump_parallax.vsh
{
	include parallax.vsh
}


_bump_parallax.fsh
{
	uniform sampler2D	u_ColorMap;
	uniform float		u_ColorMapSwizzle;
	uniform sampler2D	u_HeightMap;
	uniform float		u_HeightMapSwizzle;
	uniform sampler2D	u_NormalMap;
	uniform float		u_NormalMapSwizzle;
	uniform sampler2D	u_SpecularMap;
	uniform float		u_SpecularMapSwizzle;
	uniform	float		u_Mirror;
	uniform vec2		u_texSize;
	include	lod.inc
	#if defined(ALPHATEST)
		uniform float	u_AlphaTestValue;
	#endif
	#if defined(CLIP)
		uniform vec4	u_clipPlane;
	#endif
	in	vec2	texCoord;
	in	vec3	Tangent;
	in	vec3	Binormal;
	in	vec3	Normal;
	in	vec3	eyeSpaceNormal;
	in	vec3	eyeSpaceVert;
	in	vec4	matColor;
#if defined(FAKE_PPL)
		uniform sampler2D	u_LightMap;
		uniform float		u_useLightMap;
		uniform sampler2D	u_DeluxeMap;
		uniform float		u_useDeluxeMap;
		in	vec2		lmCoord;
		in	vec3		vertexColor;
#endif
	include Texture.inc
#ifndef CSM
		in	vec3		eyeVec;
#else
		in	vec3		eyeSpaceTangent;
		in	vec3		eyeSpaceBinormal;
		include intersect_cone_exp.inc
#endif
	include PackData.inc
	void main(void)
	{
		vec3 v_clipVertex = eyeSpaceVert;
		include clipPlane.inc

		float a;
		vec3 pt_eye;
#if defined(CSM)
		vec3 T = normalize(eyeSpaceTangent);
		vec3 B = normalize(eyeSpaceBinormal);
#endif
		vec3 N = normalize(eyeSpaceNormal);
		vec3 v = normalize(eyeSpaceVert);

		a = abs(dot(N, v));
		float Scale = sin(clamp(a, 0.0, 1.0) * 1.570796327);

#if defined(CSM)
		vec3 s;
		float df;

		// ray intersect in view direction
		a = Scale * HEIGHT_SCALE / a;
		s = vec3(a*dot(T, v), a*dot(B, v), 1.0);

		df = 0.05 * sqrt(length(fwidth(texCoord)));

		// find the distance to the actualy heightfield
		pt_eye = vec3(texCoord, 0.0);
		intersect_cone_exp (pt_eye, s, df);
#else
		include linearStepParallax.inc
#endif
		// compute the final color
		vec4 pix = Texture(u_ColorMap, pt_eye.xy, u_ColorMapSwizzle) * matColor;
		include alpha_test.inc
		vec4 spec = Texture(u_SpecularMap, pt_eye.xy, u_SpecularMapSwizzle);
		vec3 nn = normalize(Normal) * mix(1.0, -1.0, float(gl_FrontFacing)) * u_Mirror;
		mat3 tangentToWorldMatrix = mat3(normalize(Tangent), normalize(Binormal), nn);
		vec3 n = tangentToWorldMatrix * Texture(u_NormalMap, pt_eye.xy, u_NormalMapSwizzle).xyz;
#if defined(FAKE_PPL)
		if (u_useDeluxeMap != 0.0)
			nn = texture(u_DeluxeMap, lmCoord).xyz * 2.0 - 1.0;
		if (u_useLightMap != 0.0)
			PackData(pix, n, spec.rgb, nn, texture(u_LightMap, lmCoord).rgb, spec.a);
		else
			PackData(pix, n, spec.rgb, nn, vertexColor, spec.a);
#else
		PackData(pix, n, spec.rgb, spec.a);
#endif
	}
}



_bump_interpolate_parallax.vsh
{
	include parallax_interpolate.vsh
}

_bump_interpolate_parallax.fsh
{
	uniform sampler2D	u_ColorMap0;
	uniform float		u_ColorMap0Swizzle;
	uniform sampler2D	u_ColorMap1;
	uniform float		u_ColorMap1Swizzle;
	uniform sampler2D	u_HeightMap0;
	uniform float		u_HeightMap0Swizzle;
	uniform sampler2D	u_HeightMap1;
	uniform float		u_HeightMap1Swizzle;
	uniform sampler2D	u_NormalMap0;
	uniform float		u_NormalMap0Swizzle;
	uniform sampler2D	u_NormalMap1;
	uniform float		u_NormalMap1Swizzle;
	uniform sampler2D	u_SpecularMap0;
	uniform float		u_SpecularMap0Swizzle;
	uniform sampler2D	u_SpecularMap1;
	uniform float		u_SpecularMap1Swizzle;
	uniform	float		u_Mirror;
	uniform vec2		u_texSize0;
	uniform vec2		u_texSize1;
	include	lod.inc
	#if defined(CLIP)
		uniform vec4	u_clipPlane;
	#endif
	in	vec2	texCoord;
	in	vec3	Tangent;
	in	vec3	Binormal;
	in	vec3	Normal;
	in	vec3	eyeSpaceNormal;
	in	vec3	eyeSpaceVert;
#if defined(FAKE_PPL)
	uniform sampler2D	u_LightMap;
	uniform float		u_useLightMap;
	uniform sampler2D	u_DeluxeMap;
	uniform float		u_useDeluxeMap;
	in	vec2		lmCoord;
	in	vec4	vertexColor;
#else
	in	float	vertexAlpha;
#endif
	include Texture.inc
#ifndef CSM
	in	vec3	eyeVec;
#else
	in	vec3	eyeSpaceTangent;
	in	vec3	eyeSpaceBinormal;
	include intersect_cone_exp2.inc
#endif
	include PackData.inc
	void main(void)
	{
		vec3 v_clipVertex = eyeSpaceVert;
		include clipPlane.inc

		float a, alpha;
		vec3 pt_eye0, pt_eye1, sample0, sample1;

#if defined(FAKE_PPL)
		alpha = vertexColor.a * Texture(u_ColorMap0, texCoord, u_ColorMap0Swizzle).a;
#else
		alpha = vertexAlpha * Texture(u_ColorMap0, texCoord, u_ColorMap0Swizzle).a;
#endif
#if defined(CSM)
		vec3 T = normalize(eyeSpaceTangent);
		vec3 B = normalize(eyeSpaceBinormal);
#endif
		vec3 N = normalize(eyeSpaceNormal);
		vec3 v = normalize(eyeSpaceVert);

		a = abs(dot(N, v));
		float Scale = sin(clamp(a, 0.0, 1.0) * 1.570796327);

#if defined(CSM)
		vec3 s;
		float df;

		// ray intersect in view direction
		a = Scale * HEIGHT_SCALE / a;
		s = vec3(a*dot(T, v), a*dot(B, v), 1.0);

		df = 0.05 * sqrt(length(fwidth(texCoord)));

		// find the distance to the actualy heightfield
		pt_eye0 = pt_eye1 = vec3(texCoord, 0.0);
		intersect_cone_exp2 (pt_eye0, pt_eye1, u_HeightMap0, u_HeightMap1, s, df, u_HeightMap0Swizzle, u_HeightMap1Swizzle);
#else
		include linearStepParallax2.inc
#endif
		// interpolate offsets (alpha is non-parallaxed)
		pt_eye0.xy += alpha * (pt_eye1.xy - pt_eye0.xy);

		// use it to sample texture
		sample0 = Texture(u_ColorMap0, pt_eye0.xy, u_ColorMap0Swizzle).rgb;
		sample1 = Texture(u_ColorMap1, pt_eye0.xy, u_ColorMap1Swizzle).rgb;
		vec4 s0 = Texture(u_SpecularMap0, pt_eye0.xy, u_SpecularMap0Swizzle);
		vec4 s1 = Texture(u_SpecularMap1, pt_eye0.xy, u_SpecularMap1Swizzle);
		vec3 nn = normalize(Normal) * mix(1.0, -1.0, float(gl_FrontFacing)) * u_Mirror;
		mat3 tangentToWorldMatrix = mat3(normalize(Tangent), normalize(Binormal), nn);
		vec3 n0 = Texture(u_NormalMap0, pt_eye0.xy, u_NormalMap0Swizzle).xyz;
		vec3 n1 = Texture(u_NormalMap1, pt_eye0.xy, u_NormalMap1Swizzle).xyz;
		vec4 c = vec4(mix(sample0, sample1, alpha), 1.0);
		vec3 n = tangentToWorldMatrix * normalize(mix(n0, n1, alpha));
		vec4 spec = mix(s0, s1, alpha);
#if defined(FAKE_PPL)
		if (u_useDeluxeMap != 0.0)
			nn = texture(u_DeluxeMap, lmCoord).xyz * 2.0 - 1.0;
		if (u_useLightMap != 0.0)
			PackData(c, n, spec.rgb, nn, texture(u_LightMap, lmCoord).rgb, spec.a);
		else
			PackData(c, n, spec.rgb, nn, vertexColor.rgb, spec.a);
#else
		PackData(c, n, spec.rgb, spec.a);
#endif
	}
}



_bump_interpolate_parallax_at.vsh
{
	include parallax_interpolate.vsh
}


_bump_interpolate_parallax_at.fsh
{
	uniform sampler2D	u_ColorMap0;
	uniform float		u_ColorMap0Swizzle;
	uniform sampler2D	u_ColorMap1;
	uniform float		u_ColorMap1Swizzle;
	uniform sampler2D	u_HeightMap0;
	uniform float		u_HeightMap0Swizzle;
	uniform sampler2D	u_HeightMap1;
	uniform float		u_HeightMap1Swizzle;
	uniform sampler2D	u_NormalMap0;
	uniform float		u_NormalMap0Swizzle;
	uniform sampler2D	u_NormalMap1;
	uniform float		u_NormalMap1Swizzle;
	uniform sampler2D	u_SpecularMap0;
	uniform float		u_SpecularMap0Swizzle;
	uniform sampler2D	u_SpecularMap1;
	uniform float		u_SpecularMap1Swizzle;
	uniform	float		u_Mirror;
	uniform vec2		u_texSize0;
	uniform vec2		u_texSize1;
	include	lod.inc
	#if defined(CLIP)
		uniform vec4	u_clipPlane;
	#endif
	in	vec2	texCoord;
	in	vec3	Tangent;
	in	vec3	Binormal;
	in	vec3	Normal;
	in	vec3	eyeSpaceNormal;
	in	vec3	eyeSpaceVert;
#if defined(FAKE_PPL)
	uniform sampler2D	u_LightMap;
	uniform float		u_useLightMap;
	uniform sampler2D	u_DeluxeMap;
	uniform float		u_useDeluxeMap;
	in	vec2		lmCoord;
	in	vec4	vertexColor;
#else
	in	float	vertexAlpha;
#endif
	include Texture.inc
#ifndef CSM
	in	vec3	eyeVec;
#else
	in	vec3	eyeSpaceTangent;
	in	vec3	eyeSpaceBinormal;
	include intersect_cone_exp2.inc
#endif
	include PackData.inc
	void main(void)
	{
		vec3 v_clipVertex = eyeSpaceVert;
		include clipPlane.inc

		float a, alpha;
		vec3 pt_eye0, pt_eye1, sample0;
		vec4 sample1;

#if defined(FAKE_PPL)
		alpha = vertexColor.a * Texture(u_ColorMap0, texCoord, u_ColorMap0Swizzle).a;
#else
		alpha = vertexAlpha * Texture(u_ColorMap0, texCoord, u_ColorMap0Swizzle).a;
#endif
#if defined(CSM)
		vec3 T = normalize(eyeSpaceTangent);
		vec3 B = normalize(eyeSpaceBinormal);
#endif
		vec3 N = normalize(eyeSpaceNormal);
		vec3 v = normalize(eyeSpaceVert);

		a = abs(dot(N, v));
		float Scale = sin(clamp(a, 0.0, 1.0) * 1.570796327);

#if defined(CSM)
		vec3 s;
		float df;

		// ray intersect in view direction
		a = Scale * HEIGHT_SCALE / a;
		s = vec3(a*dot(T, v), a*dot(B, v), 1.0);

		df = 0.05 * sqrt(length(fwidth(texCoord)));

		// find the distance to the actualy heightfield
		pt_eye0 = pt_eye1 = vec3(texCoord, 0.0);
		intersect_cone_exp2 (pt_eye0, pt_eye1, u_HeightMap0, u_HeightMap1, s, df, u_HeightMap0Swizzle, u_HeightMap1Swizzle);
#else
		include linearStepParallax2.inc
#endif
		// interpolate offsets (alpha is non-parallaxed)
		pt_eye0.xy += alpha * (pt_eye1.xy - pt_eye0.xy);

		// use it to sample texture
		sample0 = Texture(u_ColorMap0, pt_eye0.xy, u_ColorMap0Swizzle).rgb;
		sample1 = Texture(u_ColorMap1, pt_eye0.xy, u_ColorMap1Swizzle);

		// interpolate colors
#if defined(FAKE_PPL)
		alpha = sample1.a * vertexColor.a;
#else
		alpha = sample1.a * vertexAlpha;
#endif
		vec4 s0 = Texture(u_SpecularMap0, pt_eye0.xy, u_SpecularMap0Swizzle);
		vec4 s1 = Texture(u_SpecularMap1, pt_eye0.xy, u_SpecularMap1Swizzle);
		vec3 nn = normalize(Normal) * mix(1.0, -1.0, float(gl_FrontFacing)) * u_Mirror;
		mat3 tangentToWorldMatrix = mat3(normalize(Tangent), normalize(Binormal), nn);
		vec3 n0 = Texture(u_NormalMap0, pt_eye0.xy, u_NormalMap0Swizzle).xyz;
		vec3 n1 = Texture(u_NormalMap1, pt_eye0.xy, u_NormalMap1Swizzle).xyz;
		alpha *= step(0.5, alpha);
		vec4 c = vec4(mix(sample0, sample1.rgb, alpha), 1.0);
		vec3 n = tangentToWorldMatrix * normalize(mix(n0, n1, alpha));
		vec4 spec = mix(s0, s1, alpha);
#if defined(FAKE_PPL)
		if (u_useDeluxeMap != 0.0)
			nn = texture(u_DeluxeMap, lmCoord).xyz * 2.0 - 1.0;
		if (u_useLightMap != 0.0)
			PackData(c, n, spec.rgb, nn, texture(u_LightMap, lmCoord).rgb, spec.a);
		else
			PackData(c, n, spec.rgb, nn, vertexColor.rgb, spec.a);
#else
		PackData(c, n, spec.rgb, spec.a);
#endif
	}
}
