// For standart Quake3 renderer (r_ppl = 0)

_replace.vsh
{
	uniform	mat4	u_ModelViewMatrix;
	uniform	mat4	u_ModelViewProjectionMatrix;
	uniform vec2	u_lmModifier;	// X = Mul, Y = Add
	out	vec3	v_clipVertex;
	out	vec4	color;
	out	vec2	texCoord;
	include deforms.inc
	include colorGen.inc
	include tcGen.inc
	void main(void)
	{
		mat2 dummy;
		vec4 glVertex = DeformGeometry(dummy);
		color = colorGen(glVertex.xyz, u_gen, u_parm0, u_parm1, u_parm2, u_parma0, u_parma1);
		color.rgb = color.rgb * u_lmModifier.x + u_lmModifier.y;
		tcGen(texCoord, glVertex.xyz);
		gl_Position = u_ModelViewProjectionMatrix * glVertex;
		v_clipVertex = (u_ModelViewMatrix * glVertex).xyz;
	}
}

_replace.fsh
{
	uniform sampler2D	u_ColorMap;
	uniform float		u_ColorMapSwizzle;
	uniform	vec4		u_globalFogParms;	// mode (0 = none, 1 = linear, 2 = exp), density, start, end
	uniform	vec3		u_globalFogColor;
	#if defined(ALPHATEST)
		uniform float	u_AlphaTestValue;
	#endif
	#if defined(CLIP)
		uniform vec4	u_clipPlane;
	#endif
	in	vec3		v_clipVertex;
	in	vec4		color;
	in	vec2		texCoord;
	out	vec4		fragColor;
	include Texture.inc
	void main(void)
	{
		include clipPlane.inc
		vec4 pix = color * Texture(u_ColorMap, texCoord, u_ColorMapSwizzle);
		include alpha_test.inc
		include fog.inc
		include fog_dithering.inc
		fragColor = vec4(mix(fogColor, pix.rgb, fogFactor), pix.a);
	}
}

_multitexture.vsh
{
	uniform	mat4	u_ModelViewMatrix;
	uniform	mat4	u_ModelViewProjectionMatrix;
	out	vec3	v_clipVertex;
	out	vec4	color0;
	out	vec4	color1;
	out	vec4	texCoords;
	include deforms.inc

	uniform	vec4	u_gen_t2;
	uniform	vec4	u_parm0_t2;
	uniform	vec4	u_parm1_t2;
	uniform	vec4	u_parm2_t2;
	uniform	vec4	u_parma0_t2;
	uniform	vec3	u_parma1_t2;
	uniform vec4	u_lmModifier;	// X = Mul0, Y = Add0, Z = Mul1, W = Add1
	include colorGen.inc

	include tcGen_multitexture.inc
	void main(void)
	{
		mat2 dummy;
		vec4 glVertex = DeformGeometry(dummy);
		tcGen(texCoords.xy, texCoords.zw, glVertex.xyz);
		color0 = colorGen(glVertex.xyz, u_gen, u_parm0, u_parm1, u_parm2, u_parma0, u_parma1);
		color1 = colorGen(glVertex.xyz, u_gen_t2, u_parm0_t2, u_parm1_t2, u_parm2_t2, u_parma0_t2, u_parma1_t2);
		color0.rgb = color0.rgb * u_lmModifier.x + u_lmModifier.y;
		color1.rgb = color1.rgb * u_lmModifier.z + u_lmModifier.w;
		gl_Position = u_ModelViewProjectionMatrix * glVertex;
		v_clipVertex = (u_ModelViewMatrix * glVertex).xyz;
	}
}

_multitexture.fsh
{
	uniform sampler2D	u_ColorMap0;
	uniform float		u_ColorMap0Swizzle;
	uniform sampler2D	u_ColorMap1;
	uniform float		u_ColorMap1Swizzle;
	uniform vec2		u_Blend;
	uniform	vec4		u_globalFogParms;	// mode (0 = none, 1 = linear, 2 = exp), density, start, end
	uniform	vec3		u_globalFogColor;
	#if defined(ALPHATEST)
		uniform float	u_AlphaTestValue;
	#endif
	#if defined(CLIP)
		uniform vec4	u_clipPlane;
	#endif
	in	vec3		v_clipVertex;
	in	vec4		color0;
	in	vec4		color1;
	in	vec4		texCoords;
	include Blend.inc
	include Texture.inc
	out	vec4		fragColor;
	void main(void)
	{
		include clipPlane.inc
		vec4 d = Texture(u_ColorMap0, texCoords.xy, u_ColorMap0Swizzle) * color0;
		vec4 s = Texture(u_ColorMap1, texCoords.zw, u_ColorMap1Swizzle) * color1;
		vec4 pix = Blend(s, d, u_Blend);
		include alpha_test.inc
		include fog.inc
		include fog_dithering.inc
		fragColor = vec4(mix(fogColor, pix.rgb, fogFactor), pix.a);
	}
}

_interpolate.vsh
{
	include interpolate.vsh
}

_interpolate.fsh
{
	uniform sampler2D	u_ColorMap0;
	uniform float		u_ColorMap0Swizzle;
	uniform sampler2D	u_ColorMap1;
	uniform float		u_ColorMap1Swizzle;
	uniform	vec4		u_globalFogParms;	// mode (0 = none, 1 = linear, 2 = exp), density, start, end
	uniform	vec3		u_globalFogColor;
	#if defined(CLIP)
		uniform vec4	u_clipPlane;
	#endif
	in	vec3		v_clipVertex;
	in	vec4		color;
	in	vec4		texCoords;
	vec3			sample0;
	vec4			sample1;
	float			alpha;
	out	vec4		fragColor;
	include Texture.inc
	void main(void)
	{
		include clipPlane.inc
		include fog.inc
		if (fogFactor == 0.0)
		{	// fast exit
			fragColor = vec4(u_globalFogColor, 1.0);
			return;
		}
		sample0 = Texture(u_ColorMap0, texCoords.xy, u_ColorMap0Swizzle).rgb;
		sample1 = Texture(u_ColorMap1, texCoords.zw, u_ColorMap1Swizzle);
		alpha = sample1.a * color.a;
		include fog_dithering.inc
		fragColor = vec4(mix(fogColor, color.rgb * mix(sample0, sample1.rgb, alpha), fogFactor), 1.0);
	}
}

_interpolate_at.vsh
{
	include interpolate.vsh
}

_interpolate_at.fsh
{
	uniform sampler2D	u_ColorMap0;
	uniform float		u_ColorMap0Swizzle;
	uniform sampler2D	u_ColorMap1;
	uniform float		u_ColorMap1Swizzle;
	uniform	vec4		u_globalFogParms;	// mode (0 = none, 1 = linear, 2 = exp), density, start, end
	uniform	vec3		u_globalFogColor;
	#if defined(CLIP)
		uniform vec4	u_clipPlane;
	#endif
	in	vec3		v_clipVertex;
	in	vec4		color;
	in	vec4		texCoords;
	vec3			sample0;
	vec4			sample1;
	float			alpha;
	out	vec4		fragColor;
	include Texture.inc
	void main(void)
	{
		include clipPlane.inc
		include fog.inc
		if (fogFactor == 0.0)
		{	// fast exit
			fragColor = vec4(u_globalFogColor, 1.0);
			return;
		}
		sample0 = Texture(u_ColorMap0, texCoords.xy, u_ColorMap0Swizzle).rgb;
		sample1 = Texture(u_ColorMap1, texCoords.zw, u_ColorMap1Swizzle);
		alpha = sample1.a * color.a;
		include fog_dithering.inc
		fragColor = vec4(mix(fogColor, color.rgb * mix(sample0, sample1.rgb, alpha * step(0.5, alpha)), fogFactor), 1.0);
	}
}
